@@ -15,8 +15,10 @@ import operator
 import re
 import sys
 import threading
-from collections import deque, namedtuple
+from collections import deque
+from collections import namedtuple
 from copy import deepcopy
+from inspect import isclass
 
 __all__ = [
     'BigIntegerField',
@@ -44,6 +46,7 @@ __all__ = [
     'PostgresqlDatabase',
     'prefetch',
     'PrimaryKeyField',
+    'R',
     'SqliteDatabase',
     'TextField',
     'TimeField',
@@ -91,8 +94,7 @@ except ImportError:
     except ImportError:
         mysql = None
 
-class ImproperlyConfigured(Exception):
-    pass
+class ImproperlyConfigured(Exception): pass
 
 if sqlite3 is None and psycopg2 is None and mysql is None:
     raise ImproperlyConfigured('Either sqlite3, psycopg2 or MySQLdb must be installed')
@@ -101,7 +103,6 @@ if sqlite3:
     sqlite3.register_adapter(decimal.Decimal, str)
     sqlite3.register_adapter(datetime.date, str)
     sqlite3.register_adapter(datetime.time, str)
-    sqlite3.register_converter('decimal', lambda v: decimal.Decimal(v))
 
 SQLITE_DT_FORMATS = (
     '%Y-%m-%d %H:%M:%S',
@@ -249,7 +250,6 @@ class Leaf(object):
     __mod__ = _e(OP_LIKE)
     __pow__ = _e(OP_ILIKE)
 
-
 class Expr(Leaf):
     def __init__(self, lhs, op, rhs):
         super(Expr, self).__init__()
@@ -317,6 +317,7 @@ class Clause(Leaf):
     def clone_base(self):
         return Clause(*self.pieces)
 
+Join = namedtuple('Join', ('model_class', 'join_type', 'on'))
 
 class FieldDescriptor(object):
     def __init__(self, field):
@@ -331,7 +332,6 @@ class FieldDescriptor(object):
     def __set__(self, instance, value):
         instance._data[self.att_name] = value
 
-
 class Field(Leaf):
     _field_counter = 0
     _order = 0
@@ -417,12 +417,9 @@ class Field(Leaf):
     def between(self, low, high):
         return Expr(self, OP_BETWEEN, Clause(low, R('AND'), high))
 
-
 class IntegerField(Field):
     db_field = 'int'
-
-    def coerce(self, value):
-        return int(value)
+    coerce = int
 
 class BigIntegerField(IntegerField):
     db_field = 'bigint'
@@ -436,9 +433,7 @@ class PrimaryKeyField(IntegerField):
 
 class FloatField(Field):
     db_field = 'float'
-
-    def coerce(self, value):
-        return float(value)
+    coerce = float
 
 class DoubleField(FloatField):
     db_field = 'double'
@@ -592,10 +587,7 @@ class TimeField(Field):
 
 class BooleanField(Field):
     db_field = 'bool'
-
-    def coerce(self, value):
-        return bool(value)
-
+    coerce = bool
 
 class RelationDescriptor(FieldDescriptor):
     def __init__(self, field, rel_model):
@@ -625,7 +617,6 @@ class RelationDescriptor(FieldDescriptor):
         else:
             instance._data[self.att_name] = value
 
-
 class ReverseRelationDescriptor(object):
     def __init__(self, field):
         self.field = field
@@ -636,7 +627,6 @@ class ReverseRelationDescriptor(object):
             return self.rel_model.select().where(self.field==instance.get_id())
         return self
 
-
 class ForeignKeyField(IntegerField):
     def __init__(self, rel_model, null=False, related_name=None, cascade=False, extra=None, *args, **kwargs):
         self.rel_model = rel_model
@@ -700,7 +690,7 @@ class ForeignKeyField(IntegerField):
 
 class QueryCompiler(object):
     field_map = {
-        'bigint': 'INTEGER',
+        'bigint': 'BIGINT',
         'blob': 'BLOB',
         'bool': 'SMALLINT',
         'date': 'DATE',
@@ -827,6 +817,9 @@ class QueryCompiler(object):
         elif isinstance(expr, Model):
             s = self.interpolation
             p = [expr.get_id()]
+        elif isclass(expr) and issubclass(expr, Model):
+            s = self.quote(expr._meta.db_table)
+            p = []
         elif conv and p:
             p = [conv.db_value(i) for i in p]
 
@@ -896,7 +889,8 @@ class QueryCompiler(object):
                 from_model = curr
                 to_model = join.model_class
                 if isinstance(join.on, Expr):
-                    join_expr = join.on
+                    # Clear any alias on the join expression.
+                    join_expr = join.on.clone().alias()
                 else:
                     field = from_model._meta.rel_for_model(to_model, join.on)
                     if field:
@@ -1101,6 +1095,11 @@ class QueryResultWrapper(object):
         self._result_cache = []
         self._populated = False
 
+        if meta is not None:
+            self.column_meta, self.join_meta = meta
+        else:
+            self.column_meta = self.join_meta = None
+
     def __iter__(self):
         self.__idx = 0
 
@@ -1145,61 +1144,97 @@ class QueryResultWrapper(object):
             except StopIteration:
                 break
 
-class TuplesQueryResultWrapper(QueryResultWrapper):
+class ExtQueryResultWrapper(QueryResultWrapper):
     def __init__(self, model, cursor, meta=None):
-        super(TuplesQueryResultWrapper, self).__init__(model, cursor, meta)
+        super(ExtQueryResultWrapper, self).__init__(model, cursor, meta)
         conv = []
         identity = lambda x: x
         for i in range(len(self.cursor.description)):
-            col = self.cursor.description[i][0]
-            if col in model._meta.columns:
-                field_obj = model._meta.columns[col]
-                conv.append(field_obj.python_value)
-            else:
-                conv.append(identity)
+            column = self.cursor.description[i][0]
+            func = identity
+            if column in model._meta.columns:
+                field_obj = model._meta.columns[column]
+                column = field_obj.name
+                func = field_obj.python_value
+            elif self.column_meta is not None:
+                select_column = self.column_meta[i]
+                # Special-case handling aggregations.
+                if (isinstance(select_column, Func) and
+                        isinstance(select_column.params[0], Field)):
+                    func = select_column.params[0].python_value
+            conv.append((i, column, func))
         self.conv = conv
 
+class TuplesQueryResultWrapper(ExtQueryResultWrapper):
     def process_row(self, row):
-        return tuple([self.conv[i](col) for i, col in enumerate(row)])
-
-class NaiveQueryResultWrapper(QueryResultWrapper):
-    def __init__(self, model, cursor, meta=None):
-        super(NaiveQueryResultWrapper, self).__init__(model, cursor, meta)
-
-        cols = []
-        non_cols = []
-        for i in range(len(self.cursor.description)):
-            col = self.cursor.description[i][0]
-            if col in model._meta.columns:
-                field_obj = model._meta.columns[col]
-                cols.append((i, field_obj.name, field_obj.python_value))
-            else:
-                non_cols.append((i, col))
-        self._cols = cols
-        self._non_cols = non_cols
+        return tuple([self.conv[i][2](col) for i, col in enumerate(row)])
 
+class NaiveQueryResultWrapper(ExtQueryResultWrapper):
     def process_row(self, row):
         instance = self.model()
-        for i, fname, pv in self._cols:
-            setattr(instance, fname, pv(row[i]))
-        for i, f in self._non_cols:
-            setattr(instance, f, row[i])
+        for i, column, func in self.conv:
+            setattr(instance, column, func(row[i]))
         instance.prepared()
         return instance
 
-class DictQueryResultWrapper(NaiveQueryResultWrapper):
+class DictQueryResultWrapper(ExtQueryResultWrapper):
     def process_row(self, row):
         res = {}
-        for i, fname, pv in self._cols:
-            res[fname] = pv(row[i])
-        for i, f in self._non_cols:
-            res[f] = row[i]
+        for i, column, func in self.conv:
+            res[column] = func(row[i])
         return res
 
 class ModelQueryResultWrapper(QueryResultWrapper):
-    def __init__(self, model, cursor, meta=None):
-        super(ModelQueryResultWrapper, self).__init__(model, cursor, meta)
-        self.column_meta, self.join_meta = meta
+    def __init__(self, *args, **kwargs):
+        super(ModelQueryResultWrapper, self).__init__(*args, **kwargs)
+        self.column_map, self.join_map = self.prepare()
+
+    def prepare(self):
+        column_map = []
+        join_map = []
+        models = set([self.model])
+        for i, expr in enumerate(self.column_meta):
+            attr = conv = None
+            if isinstance(expr, Field):
+                if isinstance(expr, FieldProxy):
+                    key = expr._model_alias
+                    constructor = expr.model
+                else:
+                    key = constructor = expr.model_class
+                attr = expr.name
+                conv = expr.python_value
+            else:
+                key = constructor = self.model
+                if isinstance(expr, Expr) and expr._alias:
+                    attr = expr._alias
+            column_map.append((key, constructor, attr, conv))
+            models.add(key)
+
+        joins = self.join_meta
+        stack = [self.model]
+        while stack:
+            current = stack.pop()
+            if current not in joins:
+                continue
+
+            for join in joins[current]:
+                join_model = join.model_class
+                if join_model in models:
+                    fk_field = current._meta.rel_for_model(join_model)
+                    if not fk_field:
+                        if isinstance(join.on, Expr):
+                            fk_name = join.on._alias or join.on.lhs.name
+                        else:
+                            # Patch the joined model using the name of the
+                            # database table.
+                            fk_name = join_model._meta.db_table
+                    else:
+                        fk_name = fk_field.name
+
+                    stack.append(join_model)
+                    join_map.append((current, fk_name, join_model))
+
+        return column_map, join_map
 
     def process_row(self, row):
         collected = self.construct_instance(row)
@@ -1209,63 +1244,34 @@ class ModelQueryResultWrapper(QueryResultWrapper):
         return instances[0]
 
     def construct_instance(self, row):
-        # we have columns, models, and a graph of joins to reconstruct
         collected_models = {}
-        cols = [c[0] for c in self.cursor.description]
-        for i, expr in enumerate(self.column_meta):
+        for i, (key, constructor, attr, conv) in enumerate(self.column_map):
             value = row[i]
-            if isinstance(expr, FieldProxy):
-                key = expr._model_alias # model alias
-                constructor = expr.model # instance constructor
-            elif isinstance(expr, Field):
-                key = constructor = expr.model_class
-            else:
-                key = constructor = self.model
-
             if key not in collected_models:
                 collected_models[key] = constructor()
             instance = collected_models[key]
-
-            if isinstance(expr, Field):
-                setattr(instance, expr.name, expr.python_value(value))
-            elif isinstance(expr, Expr) and expr._alias:
-                setattr(instance, expr._alias, value)
-            else:
-                setattr(instance, cols[i], value)
+            if attr is None:
+                attr = self.cursor.description[i][0]
+            if conv is not None:
+                value = conv(value)
+            setattr(instance, attr, value)
 
         return collected_models
 
     def follow_joins(self, collected):
-        joins = self.join_meta
-        stack = [self.model]
         prepared = [collected[self.model]]
-        while stack:
-            current = stack.pop()
-            if current not in joins:
-                continue
+        for (lhs, attr, rhs) in self.join_map:
+            inst = collected[lhs]
+            joined_inst = collected[rhs]
 
-            inst = collected[current]
-            for join in joins[current]:
-                if join.model_class in collected:
-                    joined_inst = collected[join.model_class]
-                    fk_field = current._meta.rel_for_model(join.model_class)
-                    if not fk_field:
-                        if isinstance(join.on, Expr):
-                            fk_field = join.on.lhs
-                        else:
-                            continue
+            if joined_inst.get_id() is None and attr in inst._data:
+                joined_inst.set_id(inst._data[attr])
 
-                    if joined_inst.get_id() is None and fk_field.name in inst._data:
-                        rel_inst_id = inst._data[fk_field.name]
-                        joined_inst.set_id(rel_inst_id)
-
-                    setattr(inst, fk_field.name, joined_inst)
-                    stack.append(join.model_class)
-                    prepared.append(joined_inst)
+            setattr(inst, attr, joined_inst)
+            prepared.append(joined_inst)
 
         return prepared
 
-Join = namedtuple('Join', ('model_class', 'join_type', 'on'))
 
 class Query(Leaf):
     require_commit = True
@@ -1397,14 +1403,16 @@ class Query(Leaf):
     def execute(self):
         raise NotImplementedError
 
-    def scalar(self, as_tuple=False):
-        row = self._execute().fetchone()
+    def scalar(self, as_tuple=False, convert=False):
+        if convert:
+            row = self.tuples().first()
+        else:
+            row = self._execute().fetchone()
         if row and not as_tuple:
             return row[0]
         else:
             return row
 
-
 class RawQuery(Query):
     def __init__(self, model, query, *params):
         self._sql = query
@@ -1449,7 +1457,6 @@ class RawQuery(Query):
     def __iter__(self):
         return iter(self.execute())
 
-
 class SelectQuery(Query):
     def __init__(self, model_class, *selection):
         super(SelectQuery, self).__init__(model_class)
@@ -1498,7 +1505,7 @@ class SelectQuery(Query):
                 accum.append(arg)
             elif isinstance(arg, ModelAlias):
                 accum.extend(arg.get_proxy_fields())
-            elif issubclass(arg, Model):
+            elif isclass(arg) and issubclass(arg, Model):
                 accum.extend(arg._meta.get_fields())
         return accum
 
@@ -1572,15 +1579,15 @@ class SelectQuery(Query):
         query._select = [aggregation]
         return query
 
-    def aggregate(self, aggregation=None):
-        return self._aggregate(aggregation).scalar()
+    def aggregate(self, aggregation=None, convert=True):
+        return self._aggregate(aggregation).scalar(convert=convert)
 
     def count(self):
         if self._distinct or self._group_by:
             return self.wrapped_count()
 
         # defaults to a count() of the primary key
-        return self.aggregate() or 0
+        return self.aggregate(convert=False) or 0
 
     def wrapped_count(self):
         clone = self.order_by()
@@ -1624,7 +1631,7 @@ class SelectQuery(Query):
 
     def execute(self):
         if self._dirty or not self._qr:
-            query_meta = None
+            query_meta = [self._select, self._joins]
             if self._tuples:
                 ResultWrapper = TuplesQueryResultWrapper
             elif self._dicts:
@@ -1632,7 +1639,6 @@ class SelectQuery(Query):
             elif self._naive or not self._joins or self.verify_naive():
                 ResultWrapper = NaiveQueryResultWrapper
             else:
-                query_meta = [self._select, self._joins]
                 ResultWrapper = ModelQueryResultWrapper
             self._qr = ResultWrapper(self.model_class, self._execute(), query_meta)
             self._dirty = False
@@ -1665,7 +1671,6 @@ class SelectQuery(Query):
         res = list(self)
         return limit == 1 and res[0] or res
 
-
 class UpdateQuery(Query):
     def __init__(self, model_class, update=None):
         self._update = update
@@ -1801,11 +1806,11 @@ class Database(object):
             self.op_overrides)
 
     def execute_sql(self, sql, params=None, require_commit=True):
+        logger.debug((sql, params))
         cursor = self.get_cursor()
         res = cursor.execute(sql, params or ())
         if require_commit and self.get_autocommit():
             self.commit()
-        logger.debug((sql, params))
         return cursor
 
     def begin(self):
@@ -1886,7 +1891,6 @@ class Database(object):
     def extract_date(self, date_part, date_field):
         return fn.EXTRACT(Clause(date_part, R('FROM'), date_field))
 
-
 class SqliteDatabase(Database):
     limit_max = -1
     op_overrides = {
@@ -1913,11 +1917,9 @@ class SqliteDatabase(Database):
     def extract_date(self, date_part, date_field):
         return fn.date_part(date_part, date_field)
 
-
 class PostgresqlDatabase(Database):
     commit_select = True
     field_overrides = {
-        'bigint': 'BIGINT',
         'blob': 'BYTEA',
         'bool': 'BOOLEAN',
         'datetime': 'TIMESTAMP',
@@ -1977,11 +1979,9 @@ class PostgresqlDatabase(Database):
         path_params = ','.join(['%s'] * len(search_path))
         self.execute_sql('SET search_path TO %s' % path_params, search_path)
 
-
 class MySQLDatabase(Database):
     commit_select = True
     field_overrides = {
-        'bigint': 'BIGINT',
         'boolean': 'BOOL',
         'decimal': 'NUMERIC',
         'double': 'DOUBLE PRECISION',
@@ -2069,13 +2069,41 @@ class transaction(object):
         return success
 
 
-class DoesNotExist(Exception):
-    pass
+class FieldProxy(Field):
+    def __init__(self, alias, field_instance):
+        self._model_alias = alias
+        self.model = self._model_alias.model_class
+        self.field_instance = field_instance
 
+    def clone_base(self):
+        return FieldProxy(self._model_alias, self.field_instance)
+
+    def __getattr__(self, attr):
+        if attr == 'model_class':
+            return self._model_alias
+        return getattr(self.field_instance, attr)
+
+class ModelAlias(object):
+    def __init__(self, model_class):
+        self.__dict__['model_class'] = model_class
+
+    def __getattr__(self, attr):
+        model_attr = getattr(self.model_class, attr)
+        if isinstance(model_attr, Field):
+            return FieldProxy(self, model_attr)
+        return model_attr
+
+    def __setattr__(self, attr, value):
+        raise AttributeError('Cannot set attributes on ModelAlias instances')
+
+    def get_proxy_fields(self):
+        return [FieldProxy(self, f) for f in self.model_class._meta.get_fields()]
+
+
+class DoesNotExist(Exception): pass
 
 default_database = SqliteDatabase('peewee.db')
 
-
 class ModelOptions(object):
     def __init__(self, cls, database=None, db_table=None, indexes=None,
                  order_by=None, primary_key=None):
@@ -2214,38 +2242,6 @@ class BaseModel(type):
 
         return cls
 
-
-class FieldProxy(Field):
-    def __init__(self, alias, field_instance):
-        self._model_alias = alias
-        self.model = self._model_alias.model_class
-        self.field_instance = field_instance
-
-    def clone_base(self):
-        return FieldProxy(self._model_alias, self.field_instance)
-
-    def __getattr__(self, attr):
-        if attr == 'model_class':
-            return self._model_alias
-        return getattr(self.field_instance, attr)
-
-class ModelAlias(object):
-    def __init__(self, model_class):
-        self.__dict__['model_class'] = model_class
-
-    def __getattr__(self, attr):
-        model_attr = getattr(self.model_class, attr)
-        if isinstance(model_attr, Field):
-            return FieldProxy(self, model_attr)
-        return model_attr
-
-    def __setattr__(self, attr, value):
-        raise AttributeError('Cannot set attributes on ModelAlias instances')
-
-    def get_proxy_fields(self):
-        return [FieldProxy(self, f) for f in self.model_class._meta.get_fields()]
-
-
 class Model(with_metaclass(BaseModel)):
     def __init__(self, *args, **kwargs):
         self._data = self._meta.get_default_dict()
@@ -2325,7 +2321,11 @@ class Model(with_metaclass(BaseModel)):
             db.create_sequence(pk.sequence)
 
         db.create_table(cls)
+        cls._create_indexes()
 
+    @classmethod
+    def _create_indexes(cls):
+        db = cls._meta.database
         for field_name, field_obj in cls._meta.fields.items():
             if isinstance(field_obj, ForeignKeyField):
                 db.create_foreign_key(cls, field_obj)
